# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:38:04+00:00

from __future__ import annotations

from pydantic import BaseModel, Extra


class Websocket(BaseModel):
    class Config:
        extra = Extra.forbid

    subscription: list[str] | None = None
    ignore: list[str] | None = None
    full: bool | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:35:34+00:00

from __future__ import annotations

import uuid
from enum import Enum
from typing import Any

from pydantic import BaseModel, Extra, Field, conint, constr


class Operation(Enum):
    async_ = 'async'
    sync = 'sync'
    intersect = 'intersect'


class Method(Enum):
    POST = 'POST'
    GET = 'GET'
    DELETE = 'DELETE'
    PUT = 'PUT'
    HEAD = 'HEAD'
    PATCH = 'PATCH'


class InputItem(BaseModel):
    class Config:
        extra = Extra.forbid

    method: Method
    url: str
    query: list[str] | None = None
    body: Any | None = None


class Action(Enum):
    stop = 'stop'
    restart = 'restart'


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Repeat(BaseModel):
    class Config:
        extra = Extra.allow

    name: str | None = None
    interval: conint(ge=0) | None = None
    operation:  Operation | None = None
    input: list[InputItem]
    action: Action | None = None
    meta: Meta | None = Field(None, title='meta-2.1')



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:34:04+00:00

from __future__ import annotations

import uuid
from enum import Enum
from typing import Any

from pydantic import BaseModel, Extra, Field, constr


class FromItem(BaseModel):
    class Config:
        extra = Extra.forbid

    type: str
    filter: list | None = None
    automatically_accept: bool | None = None


class Application(BaseModel):
    class Config:
        extra = Extra.forbid

    sharing: bool | None = None
    permitted: list[uuid.UUID] | None = None


class Method(BaseModel):
    create: bool | None = None
    update: bool | None = None
    retrieve: bool | None = None
    delete: bool | None = None


class State(BaseModel):
    class Config:
        extra = Extra.forbid

    min_state: float | None = None
    max_state: float | None = None


class Installation(BaseModel):
    class Config:
        extra = Extra.forbid

    shareable: bool | None = None


class File(BaseModel):
    background: bool | None = None


class AclAttribute(Enum):
    parent_name = 'parent_name'


class Restriction(BaseModel):
    class Config:
        extra = Extra.forbid

    child: str | None = None
    create: list[str] | None = None
    application:  Application | None = None
    method:  Method | None = None
    time: dict[str, Any] | None = None
    state:  State | None = None
    installation:  Installation | None = None
    file:  File | None = None
    acl_attributes: list[AclAttribute] | None = None


class ToItem(BaseModel):
    class Config:
        extra = Extra.forbid

    type: str
    filter: list | None = None
    restriction:  Restriction | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class PermissionRule(BaseModel):
    class Config:
        extra = Extra.forbid

    apply_to_old: bool | None = None
    from_: list[FromItem] | None = Field(None, alias='from')
    to: list[ToItem] | None = None
    meta: Meta | None = Field(None, title='meta-2.1')



from ..path_helpers import Wrapper


class Meta:
    __path__ = 'state.meta'
    overrite_list = [
        'id',
        'trace',
        'redirect',
        'icon',
        'tag',
        'tag_by_user',
        'name_by_user',
        'historical',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class State:
    __path__ = 'state'
    overrite_list = [
        'timestamp',
        'data',
        'type',
        'status',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    meta = Meta



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:28:39+00:00

from __future__ import annotations

from enum import Enum

from pydantic import BaseModel, Extra


class Bucket(Enum):
    default = 'default'
    user = 'user'


class Couchbase(BaseModel):
    class Config:
        extra = Extra.forbid

    bucket:  Bucket | None = None
    id: list[str] | None = None
    metadata: bool | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:34:48+00:00

from __future__ import annotations

import uuid
from typing import Any
from pydantic import BaseModel, Extra, constr


class DriverItem(BaseModel):
    class Config:
        extra = Extra.forbid

    type: str | None = None
    config: dict[str, Any] | Any | None = None
    mapping: dict[str, Any] | None = None


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Prototype(BaseModel):
    class Config:
        extra = Extra.allow

    creator_id: uuid.UUID | None = None
    model_id: uuid.UUID | None = None
    prototype_id: uuid.UUID | None = None
    name: str | None = None
    model: dict[str, Any] | None = None
    driver: list[DriverItem] | None = None
    target: str | None = None
    deploy: bool | None = None
    built: bool | None = None
    meta:  MetaItem | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:31:48+00:00

from __future__ import annotations

from enum import Enum
from typing import Any
from typing import Literal

from pydantic import BaseModel, Extra, Field, conint


class WappstoVersion(Enum):
    V2_0 = "2.0"
    V2_1 = "2.1"


class ChildItem(BaseModel):
    type: str | None = None
    version: WappstoVersion | None = None


class WarningItem(BaseModel):
    message: str | None = None
    data: dict[str, Any] | None = None
    code: int | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.forbid

    type:  Literal['idlist']
    version: WappstoVersion | None = None

    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    warning: list[WarningItem] | None = None


class Idlist(BaseModel):
    class Config:
        extra = Extra.forbid

    id: list[str]
    more: bool
    limit: conint(ge=1, multiple_of=1)
    count: conint(ge=0, multiple_of=1)
    data: dict[str, Any] | None = None
    child: list[ChildItem]
    meta: Meta = Field(..., title='meta_list-2.1')



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:37:40+00:00

from __future__ import annotations

from pydantic import BaseModel


class TransferUser(BaseModel):
    username: str
    password: str | None = None
    documents: list[str] | None = None
    tables: list[str] | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:26:55+00:00

from __future__ import annotations

import uuid
from enum import Enum

from pydantic import BaseModel, Extra, constr


class LevelEnum(Enum):
    founder = 'founder'
    admin = 'admin'
    sales = 'sales'
    empty = 'empty'
    custom = 'custom'


class Acces(BaseModel):
    pages: list[str] | None = None
    tabs: list[str] | None = None


class AclItem(BaseModel):
    change_manufacturer: bool | None = None
    change_owner: bool | None = None
    share_object: bool | None = None


class AdminItem(BaseModel):
    handle_admin: bool | None = None


class ApplicationItem(BaseModel):
    create_admin_application: bool | None = None
    delete_published: bool | None = None
    disable_and_reenable: bool | None = None
    handle_application_product: bool | None = None
    handle_publishing: bool | None = None
    retrieve_oauth_secrets: bool | None = None


class BillingItem(BaseModel):
    change_user_subscription: bool | None = None
    handle_subscription: bool | None = None
    handle_tax: bool | None = None
    show_stripe: bool | None = None


class CommunicationItem(BaseModel):
    create_translation: bool | None = None
    delete_translation: bool | None = None
    publish_translation: bool | None = None
    send_email: bool | None = None
    send_notification: bool | None = None
    send_sms: bool | None = None
    update_email: bool | None = None
    update_sms: bool | None = None
    update_translation: bool | None = None


class CustomerItem(BaseModel):
    handle_customer: bool | None = None
    handle_seller: bool | None = None
    list_customer: bool | None = None
    list_seller: bool | None = None


class CouchbaseItem(BaseModel):
    access_user_bucket: bool | None = None
    delete: bool | None = None
    retrieve: bool | None = None
    save: bool | None = None
    retrieve_meta: bool | None = None


class Documentation(BaseModel):
    access_admin_documentation: bool | None = None


class InstallationItem(BaseModel):
    restart_with_new_process: bool | None = None
    restart_with_new_user: bool | None = None
    retrieve_oauth_secret_token: bool | None = None
    stop_background: bool | None = None
    log_background: bool | None = None


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class MicroItem(BaseModel):
    read_environment: bool | None = None
    retrieve_micro: bool | None = None
    show_environment: bool | None = None
    show_grafana: bool | None = None
    update_environment: bool | None = None


class NetworkItem(BaseModel):
    contact_directly: bool | None = None
    update_creator: bool | None = None
    handle_firmware: bool | None = None


class RecoveryItem(BaseModel):
    recover_background: bool | None = None
    recover_cleanup: bool | None = None
    recover_repeat: bool | None = None


class Service(BaseModel):
    cockroach: bool | None = None
    full_search: bool | None = None
    n1ql: bool | None = None
    nats: bool | None = None


class SessionItem(BaseModel):
    write_permission: bool | None = None


class SupportItem(BaseModel):
    delete_historical: bool | None = None
    handle_cleanup_historical: bool | None = None
    handle_delayed_state: bool | None = None
    handle_pending: bool | None = None


class TransferItem(BaseModel):
    promote_application: bool | None = None
    promote_email_and_sms: bool | None = None
    promote_subscription: bool | None = None
    promote_translation: bool | None = None
    transfer_user: bool | None = None


class UiadminItem(BaseModel):
    show_create_object_button: bool | None = None
    show_delete_object_button: bool | None = None
    show_update_object_button: bool | None = None


class UserItem(BaseModel):
    ban_user: bool | None = None
    change_password: bool | None = None
    change_username: bool | None = None
    create_new_user: bool | None = None
    delete_user: bool | None = None
    reset_captcha: bool | None = None
    verify_communication: bool | None = None


class WalletItem(BaseModel):
    change_points: bool | None = None
    correct_points: bool | None = None
    reset_points: bool | None = None


class AdminFeature(BaseModel):
    class Config:
        extra = Extra.allow

    level:  LevelEnum | None = None
    access:  Acces | None = None
    acl:  AclItem | None = None
    admin:  AdminItem | None = None
    application:  ApplicationItem | None = None
    billing:  BillingItem | None = None
    communication:  CommunicationItem | None = None
    customer:  CustomerItem | None = None
    couchbase:  CouchbaseItem | None = None
    documentation:  Documentation | None = None
    installation:  InstallationItem | None = None
    meta:  MetaItem | None = None
    micro:  MicroItem | None = None
    network:  NetworkItem | None = None
    recovery:  RecoveryItem | None = None
    services:  Service | None = None
    session:  SessionItem | None = None
    support:  SupportItem | None = None
    transfer:  TransferItem | None = None
    uiadmin:  UiadminItem | None = None
    user:  UserItem | None = None
    wallet:  WalletItem | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:26:45+00:00

from __future__ import annotations

import uuid
from pydantic import BaseModel, Extra, constr


class Meta(BaseModel):
    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    trace: str | None = None
    accept_manufacturer_as_owner: bool | None = None
    accept_test_mode: bool | None = None


class AddNotOwned(BaseModel):
    class Config:
        extra = Extra.forbid

    meta:  Meta | None = None



import datetime

from enum import Enum
from pydantic import BaseModel
from pydantic import constr
from pydantic import UUID4

from typing import Any

from ..core.utils import timestamp


class WappstoEnv(Enum):
    """Wappstos difference environments."""
    PROD = 'prod'
    QA = 'qa'
    DEV = 'dev'
    STAGING = 'staging'


class WappstoService(Enum):
    """Wappstos difference Services."""
    ACL = "acl"
    ADD_NOT_OWNED = "add_not_owned"
    ADMIN_FEATURE = "admin_feature"
    ALERT = "alert"
    ANALYTICS = "analytics"
    ANALYTICS_MODEL = "analytics_model"
    API = "api"
    APPLICATION = "application"
    APPLICATION_PRODUCT = "application_product"
    ASYNCHRONOUS = "asynchronous"
    ATTRIBUTELIST = "attributelist"
    AZURE = "azure"
    BACKGROUND = "background"
    BAN_LIST = "ban_list"
    BUS_STREAM = "bus_stream"
    CLEANUP_HISTORICAL = "cleanup_historical"
    COCKROACH = "cockroach"
    CONSOLE = "console"
    COUCHBASE = "couchbase"
    COUNTER = "counter"
    CREATOR = "creator"
    CUSTOMER = "customer"
    CUSTOMER_INVOICE = "customer_invoice"
    DASHBOARD = "dashboard"
    DATA = "data"
    DEBUGGER = "debugger"
    DELETELIST = "deletelist"
    DEVICE = "device"
    DOWNLOAD = "download"
    EMAIL = "email"
    EMAIL_DRAFT = "email_draft"
    EMAIL_SEND = "email_send"
    EMAIL_TRANSLATION = "email_translation"
    ERROR = "error"
    EVENTLOG = "eventlog"
    EVENTSTREAM = "eventstream"
    EXPECTED_STATE = "expected_state"
    EXTSYNC = "extsync"
    EXTSYNC_RESPONSE = "extsync_response"
    FETCH = "fetch"
    FILE = "file"
    FILTER = "filter"
    FIRMWARE = "firmware"
    FITE = "fite"
    HOMEPAGE = "homepage"
    HTTPRESPONSE = "httpresponse"
    ICON = "icon"
    IDLIST = "idlist"
    INSTALLATION = "installation"
    INVOICE = "invoice"
    LOG = "log"
    LOG_ZIP = "log_zip"
    MDA = "mda"
    META = "meta"
    METRICS = "metrics"
    MICRO = "micro"
    MODBUS = "modbus"
    MQTT = "mqtt"
    N1QL = "n1ql"
    NATS = "nats"
    NETWORK = "network"
    NOTIFICATION = "notification"
    OAUTH = "oauth"
    OAUTH_CLIENT = "oauth_client"
    OAUTH_CONNECT = "oauth_connect"
    OAUTH_EXTERNAL = "oauth_external"
    ONTOLOGY = "ontology"
    PARSER = "parser"
    PENDING_USER = "pending_user"
    PERMISSION = "permission"
    PERMISSION_GROUP = "permission_group"
    PERMISSION_RULE = "permission_rule"
    POINT = "point"
    PRICE = "price"
    PROCESS = "process"
    PRODUCT = "product"
    PROMOTE = "promote"
    PROTOTYPE = "prototype"
    PURSE = "purse"
    PURSE_APPLICATION = "purse_application"
    QUERY = "query"
    RECOVERY = "recovery"
    RECOVERY_CLEANUP = "recovery_cleanup"
    RECOVERY_REPEAT = "recovery_repeat"
    REGISTER = "register"
    REPEAT = "repeat"
    RPC_ERROR = "rpc_error"
    RPC_REQUEST = "rpc_request"
    RPC_RESULT = "rpc_result"
    SCHEMA = "schema"
    SEARCH = "search"
    SELLER = "seller"
    SESSION = "session"
    SMS = "sms"
    SMS_DRAFT = "sms_draft"
    SMS_SEND = "sms_send"
    STATE = "state"
    STATUS = "status"
    STORAGE = "storage"
    STREAM = "stream"
    STRIPE = "stripe"
    SUBSCRIPTION = "subscription"
    SUBUSER = "subuser"
    SUMMARY = "summary"
    TAX = "tax"
    TRANSFER_BILLING = "transfer_billing"
    TRANSFER_EMAIL = "transfer_email"
    TRANSFER_TRANSLATION = "transfer_translation"
    TRANSFER_USER = "transfer_user"
    USER = "user"
    VALUE = "value"
    VERSION = "version"
    VIRTUALUSER = "virtualuser"
    WEBSOCKET = "websocket"
    WIDGET = "widget"


class WappstoVersion(Enum):
    V2_0 = "2.0"
    V2_1 = "2.1"


class ApiMetaTypes(Enum):
    # TODO: Add all services from WappstoService to this.
    idlist = "idlist"
    deletelist = "deletelist"


class Deletion(str, Enum):
    PENDING = 'pending'
    FAILED = 'failed'


class Assigned(str, Enum):
    UNASSIGNED = 'unassigned'


class ApiMetaInfo(BaseModel):
    type: ApiMetaTypes  # Merge with MetaAPIData?
    version: WappstoVersion


class WarningItem(BaseModel):
    message: str | None = None
    data: dict[str, Any] | None = None
    code: int | None = None


class BaseMeta(BaseModel):  # Base Meta
    id: UUID4 | None = None
    # NOTE: Set in the children-class.
    # #  type: Optional[WappstoMetaType] = None
    version: WappstoVersion | None = None

    manufacturer: UUID4 | None = None
    owner: UUID4 | Assigned | None = None
    parent: UUID4 | None = None

    created: datetime.datetime | None = None
    updated: datetime.datetime | None = None
    changed: datetime.datetime | None = None

    application: UUID4 | None = None
    deletion: Deletion | None = None
    deprecated: bool | None = None

    iot: bool | None = None
    revision: int | None = None
    size: int | None = None
    path: str | None = None

    oem: str | None = None
    accept_manufacturer_as_owner: bool | None = None
    redirect: constr(  # type: ignore
        regex=r'^[0-9a-zA-Z_-]+$',  # noqa: F722
        min_length=1,
        max_length=200
    ) | None = None

    error: UUID4 | None = None
    warning: list[WarningItem] | None = None
    trace: str | None = None

    set: list[UUID4] | None = None
    contract: list[UUID4] | None = None

    historical: bool | None = None

    class Config:
        json_encoders = {
            datetime: timestamp
        }



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:33:57+00:00

from __future__ import annotations

import uuid
from enum import Enum

from pydantic import BaseModel, Extra, Field, conint


class IdEnum(Enum):
    global_ = 'global'


class Permission(BaseModel):
    class Config:
        extra = Extra.forbid

    deeper: bool | None = None
    id: list[uuid.UUID | IdEnum] | None = None
    global_: bool | None = Field(None, alias='global')
    installation: bool | None = None
    propagate: bool | None = None
    temporary: bool | None = None
    expand: conint(ge=0, multiple_of=1) | None = None
    offset: conint(ge=0, multiple_of=1) | None = None
    limit: conint(ge=0, multiple_of=1) | None = None
    verbose: bool | list[str] | None = None
    prev_notification: uuid.UUID | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:32:07+00:00

from __future__ import annotations

import uuid
from typing import Any
from pydantic import BaseModel, Extra, constr


class RestartItem(BaseModel):
    class Config:
        extra = Extra.forbid

    new_process: bool | None = None
    new_user: bool | None = None
    stop_background: bool | None = None


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class OauthItem(BaseModel):
    class Config:
        extra = Extra.forbid

    client_id: uuid.UUID | None = None
    name: str | None = None
    expires: bool | None = None
    expired: bool | None = None
    active: bool | None = None
    installation_id: uuid.UUID | None = None
    meta:  MetaItem | None = None


class OauthConnectItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str | None = None
    api: uuid.UUID | None = None
    installation: uuid.UUID | None = None
    token: str | None = None
    secret_token: str | None = None
    params: dict[str, Any] | None = None
    meta:  MetaItem | None = None
    access_token_creation: dict[str, Any] | None = None


class Installation(BaseModel):
    class Config:
        extra = Extra.allow

    application: uuid.UUID | None = None
    title: str | None = None
    upgrade: bool | None = None
    ignore_this_notification: str | None = None
    extsync: bool | None = None
    permit_to_send_email: bool | None = None
    permit_to_send_sms: bool | None = None
    no_start_background: bool | None = None
    restart:  RestartItem | None = None
    application_product: uuid.UUID | None = None
    meta:  MetaItem | None = None
    oauth: list[OauthItem | uuid.UUID] | None = None
    oauth_connect: list[OauthConnectItem | uuid.UUID] | None = None



from ..path_helpers import Wrapper


class ProviderItem():
    __path__ = 'user.provideritem'
    overrite_list = [
        'name',
        'picture',
        'type',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class OtherEmailItem:
    __path__ = 'user.otheremailitem'
    overrite_list = [
        'contact',
        'status',
        'last_update',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class OtherSm:
    __path__ = 'user.othersm'
    overrite_list = [
        'contact',
        'status',
        'last_update',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Ban:
    __path__ = 'user.ban'
    overrite_list = [
        'type',
        'begin_ban',
        'end_ban',
        'motivation',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Meta:
    __path__ = 'user.meta'
    overrite_list = [
        'id',
        'trace',
        'redirect',
        'icon',
        'tag',
        'tag_by_user',
        'name_by_user',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class User:
    __path__: str = 'user'
    overrite_list = [
        'first_name',
        'last_name',
        'email',
        'phone',
        'name',
        'role',
        'nickname',
        'language',
        'friend',
        'blocked',
        'verified_email',
        'verified_sms',
        'admin',
        'founder',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    provider = ProviderItem
    other_email = OtherEmailItem
    other_sms = OtherSm
    ban = Ban
    meta = Meta



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:34:08+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Literal

from pydantic import BaseModel, Extra, Field, conint, constr


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Actual(BaseModel):
    last_update: datetime.datetime | None = None
    point: int | None = None
    document: int | None = None
    log_row: int | None = None
    traffic_in: int | None = None
    traffic_out: int | None = None
    iot_traffic_in: int | None = None
    iot_traffic_out: int | None = None
    stream_traffic: int | None = None
    file: int | None = None
    request: int | None = None
    request_time: int | None = None
    sms: int | None = None


class UserDailyLimit(BaseModel):
    class Config:
        extra = Extra.forbid

    point: conint(ge=0) | Literal['dynamic'] | None = None
    document: conint(ge=0) | None = None
    log_row: conint(ge=0) | None = None
    traffic: conint(ge=0) | None = None
    iot_traffic: conint(ge=0) | None = None
    stream_traffic: conint(ge=0) | None = None
    file: conint(ge=0) | None = None
    request: conint(ge=0) | None = None
    request_time: conint(ge=0) | None = None
    sms: conint(ge=0) | None = None


class NetworkDailyLimit(BaseModel):
    class Config:
        extra = Extra.forbid

    point: conint(ge=0) | Literal['dynamic'] | None = None
    iot_traffic: conint(ge=0) | None = None
    request: conint(ge=0) | None = None
    request_time: conint(ge=0) | None = None


class Point(BaseModel):
    class Config:
        extra = Extra.allow

    meta: Meta | None = Field(None, title='meta-2.1')
    actual:  Actual | None = None
    user_daily_limit: UserDailyLimit | None = None
    network_daily_limit: NetworkDailyLimit | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:37:10+00:00

from __future__ import annotations

import uuid
from enum import Enum
from pydantic import BaseModel, Extra, Field, conint, constr


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Type(Enum):
    private = 'private'
    public = 'public'
    enterprise = 'enterprise'


class MaxNumberNetworkEnum(Enum):
    unlimited = 'unlimited'


class BasePointEnum(Enum):
    unlimited = 'unlimited'


class BaseSm(Enum):
    unlimited = 'unlimited'


class Interval(Enum):
    day = 'day'
    week = 'week'
    month = 'month'
    year = 'year'
    never = 'never'


class RecurringPoint(BaseModel):
    class Config:
        extra = Extra.forbid

    interval: Interval
    interval_count: conint(ge=1)


class Interval1(Enum):
    day = 'day'
    week = 'week'
    month = 'month'
    year = 'year'


class Recurring(BaseModel):
    class Config:
        extra = Extra.forbid

    interval: Interval1
    interval_count: conint(ge=1)


class Tax(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None


class PriceItem(BaseModel):
    class Config:
        extra = Extra.allow

    meta: Meta | None = Field(None, title='meta-2.1')
    name: str | None = None
    unchangeable: bool | None = None
    active: bool | None = None
    free: bool | None = None
    currency: str | None = None
    amount: int | None = None
    prepaid_extra_cost: int | None = None
    topup_extra_cost: int | None = None
    recurring: Recurring | None = None
    tax: Tax | None = None


class Subscription(BaseModel):
    class Config:
        extra = Extra.allow

    meta: Meta | None = Field(None, title='meta-2.1')
    unit_label: constr(min_length=2, max_length=12) | None = None
    description: str | None = None
    name: str
    active: bool | None = None
    free: bool
    change_immediately: bool | None = None
    images: list[str] | None = None
    type: Type
    user: list | None = None
    max_number_network: MaxNumberNetworkEnum | conint(ge=0)
    extra_network: conint(ge=0) | None = None
    base_point: BasePointEnum | conint(ge=0)
    prepaid_point: conint(ge=0) | None = None
    topup_point: conint(ge=0) | None = None
    base_sms: BaseSm | conint(ge=0) | None = None
    prepaid_sms: conint(ge=0) | None = None
    topup_sms: conint(ge=0) | None = None
    recurring_point: RecurringPoint | None = None
    price: list[PriceItem | uuid.UUID] | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:31:27+00:00

from __future__ import annotations

from typing import Any

from pydantic import BaseModel, Extra, Field, constr


class Debug(BaseModel):
    file_line: str | None = None
    debug: Any | None = None
    backtrace: list[str] | None = None


class WarningItem(BaseModel):
    message: str | None = None
    data: dict[str, Any] | None = None
    code: int | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.forbid

    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    warning: list[WarningItem] | None = None


class Httpresponse(BaseModel):
    class Config:
        extra = Extra.forbid

    message: str
    data: dict[str, Any] | None = None
    code: int
    service: str
    internal_code: str | None = None
    debug:  Debug | None = None
    meta: Meta = Field(..., title='meta_list-2.1')



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:34:00+00:00

from __future__ import annotations

import uuid

from pydantic import BaseModel, Extra, constr


class UserItem(BaseModel):
    id: uuid.UUID | None = None
    email: constr(
        regex=r'(?:[a-zA-Z0-9!#$%&\'*+/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&\'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?\.)+[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?|\[(?:(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9]))\.){3}(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9])|[a-zA-Z0-9-]*[a-zA-Z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])'
    ) | None = None
    nickname: str | None = None
    role: str | None = None


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class PermissionGroup(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str | None = None
    user: list[UserItem] | None = None
    meta:  MetaItem | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:30:41+00:00

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, conint


class Extsync(BaseModel):
    timeout: Optional[conint(ge=1, le=60)] = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:31:15+00:00

from __future__ import annotations

import datetime
import uuid
from pydantic import BaseModel, Extra, constr


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class LogItem(BaseModel):
    type: str | None = None
    timestamp: datetime.datetime | None = None
    message: str | None = None


class Path(BaseModel):
    name: str | None = None
    path: str | None = None
    firmware: uuid.UUID | None = None
    unpack: bool | None = None


class Firmware(BaseModel):
    class Config:
        extra = Extra.forbid

    meta:  MetaItem | None = None
    log: list[LogItem] | None = None
    version_id: str | None = None
    paths: list[Path] | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:34:09+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum

from pydantic import BaseModel, Extra, conint, constr


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class ActualItem(BaseModel):
    last_update: datetime.datetime | None = None
    point: int | None = None
    document: int | None = None
    log_row: int | None = None
    traffic_in: int | None = None
    traffic_out: int | None = None
    iot_traffic_in: int | None = None
    iot_traffic_out: int | None = None
    stream_traffic: int | None = None
    file: int | None = None
    request: int | None = None
    request_time: int | None = None
    sms: int | None = None


class DynamicLimitEnum(Enum):
    dynamic = 'dynamic'


class DynamicLimit(BaseModel):
    __root__: conint(ge=0) | DynamicLimitEnum


class UserDailyLimitItem(BaseModel):
    class Config:
        extra = Extra.forbid

    point: DynamicLimit | None = None
    document: conint(ge=0) | None = None
    log_row: conint(ge=0) | None = None
    traffic: conint(ge=0) | None = None
    iot_traffic: conint(ge=0) | None = None
    stream_traffic: conint(ge=0) | None = None
    file: conint(ge=0) | None = None
    request: conint(ge=0) | None = None
    request_time: conint(ge=0) | None = None
    sms: conint(ge=0) | None = None


class NetworkDailyLimitItem(BaseModel):
    class Config:
        extra = Extra.forbid

    point: DynamicLimit | None = None
    iot_traffic: conint(ge=0) | None = None
    request: conint(ge=0) | None = None
    request_time: conint(ge=0) | None = None


class Point(BaseModel):
    class Config:
        extra = Extra.allow

    meta:  MetaItem | None = None
    actual:  ActualItem | None = None
    user_daily_limit:  UserDailyLimitItem | None = None
    network_daily_limit:  NetworkDailyLimitItem | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:36:10+00:00

from __future__ import annotations

import uuid
from pydantic import BaseModel, Extra, Field, constr


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Information(BaseModel):
    company: str | None = None
    street: str | None = None
    post_number: str | None = None
    city: str | None = None
    country: str | None = None
    tax_code: str | None = None
    tax_code_type: str | None = None
    email: str | None = None
    phone: str | None = None


class Seller(BaseModel):
    class Config:
        extra = Extra.forbid

    meta: Meta | None = Field(None, title='meta-2.1')
    name: str | None = None
    archived: bool | None = None
    timestamp_format: str | None = None
    information:  Information | None = None
    bank: str | None = None
    iban: str | None = None
    swift: str | None = None
    regnr: str | None = None
    kontonr: str | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:32:16+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Literal

from pydantic import BaseModel, Extra, Field, confloat, conint, constr


class MethodEnum(Enum):
    retrieve = 'retrieve'
    update = 'update'
    add = 'add'
    delete = 'delete'


class Conjunction(Enum):
    and_ = 'and'
    or_ = 'or'


class WrongObject(Enum):
    expanded = 'expanded'
    compact = 'compact'
    hidden = 'hidden'


class Schema(Enum):
    create = 'create'
    update = 'update'
    output = 'output'
    query = 'query'


class AclAttribute(Enum):
    parent_name = 'parent_name'


class DataOperationEnum(Enum):
    bigger = 'bigger'
    smaller = 'smaller'
    equal = 'equal'
    bigger_and_equal = 'bigger_and_equal'
    smaller_and_equal = 'smaller_and_equal'
    not_equal = 'not_equal'


class DataType(Enum):
    int = 'int'
    string = 'string'


class Order(Enum):
    ascending = 'ascending'
    descending = 'descending'


class Type(Enum):
    request = 'request'
    application = 'application'
    state = 'state'
    user = 'user'
    online_iot = 'online_iot'
    geo = 'geo'
    total_online_iot = 'total_online_iot'
    active_user = 'active_user'
    active_network = 'active_network'
    highest_user = 'highest_user'
    price = 'price'
    subscription = 'subscription'


class TypeTimestamp(Enum):
    internal = 'internal'
    external = 'external'


class GroupBy(Enum):
    year = 'year'
    quarter = 'quarter'
    month = 'month'
    week = 'week'
    day = 'day'
    hour = 'hour'
    minute = 'minute'
    second = 'second'
    millisecond = 'millisecond'
    microsecond = 'microsecond'
    dow = 'dow'
    doy = 'doy'


class TimeAggregation(Enum):
    date_trunc = 'date_trunc'
    extract = 'extract'


class Operation(Enum):
    arbitrary = 'arbitrary'
    array_agg = 'array_agg'
    avg = 'avg'
    mean = 'mean'
    count = 'count'
    geometric_mean = 'geometric_mean'
    max = 'max'
    min = 'min'
    sqrdiff = 'sqrdiff'
    stddev = 'stddev'
    sum = 'sum'
    variance = 'variance'
    harmonic_mean = 'harmonic_mean'
    first = 'first'
    last = 'last'
    count_distinct = 'count_distinct'
    median = 'median'
    percentile = 'percentile'
    lower_quartile = 'lower_quartile'
    upper_quartile = 'upper_quartile'
    mode = 'mode'
    var_pop = 'var_pop'
    stddev_pop = 'stddev_pop'


class Log(BaseModel):
    class Config:
        extra = Extra.forbid

    id: list[uuid.UUID] | None = None
    offset: conint(ge=0, multiple_of=1) | None = None
    limit: conint(ge=0, multiple_of=1) | None = None
    manufacturer: bool | None = None
    owned: bool | None = None
    accepted: bool | None = None
    shared_with: list[uuid.UUID] | None = None
    not_shared_with: list[uuid.UUID] | None = None
    has_parent: bool | None = None
    version: list[constr(regex=r'\d+(\.\d+)*')] | None = None
    method: list[MethodEnum] | None = None
    deep: conint(ge=1, multiple_of=1) | None = None
    global_: bool | None = Field(None, alias='global')
    full_search: bool | None = None
    from_last: bool | None = None
    order_by: str | None = None
    permission_group: list[Literal['all'] | uuid.UUID] | None = None
    clean: bool | None = None
    conjunction:  Conjunction | None = None
    count: bool | None = None
    asynchronous: bool | uuid.UUID | None = None
    propagate: bool | None = None
    acl_code: uuid.UUID | None = None
    expand: conint(ge=0, multiple_of=1) | None = None
    quantity: int | Literal['all'] | None = None
    message: str | None = None
    opt_attr: list[str] | None = None
    opt_value: list[str] | None = None
    verbose: bool | list[str] | None = None
    compact: list[str] | None = None
    fetch: bool | None = None
    go_internal: bool | None = None
    expand_from_bottom: bool | None = None
    wrong_object:  WrongObject | None = None
    cache: conint(ge=1, le=86400) | None = None
    cache_refresh: bool | None = None
    csv_name: str | None = None
    csv_separator: str | None = None
    prev_notification: uuid.UUID | None = None
    schema_: Schema | None = Field(None, alias='schema')
    test_mode: bool | None = None
    acl_attributes: list[AclAttribute] | None = None
    start: datetime.datetime | None = None
    end: datetime.datetime | None = None
    data_value: list[str | float] | None = None
    data_operation: list[DataOperationEnum] | None = None
    data_type:  DataType | None = None
    timezone: str | None = None
    request: list[str] | None = None
    geo_precision: int | None = None
    time_precision: int | None = None
    timestamp_in_seconds: bool | None = None
    timestamp_format: str | None = None
    user_id: list[uuid.UUID] | None = None
    modifier_id: list[uuid.UUID] | None = None
    network_id: list[uuid.UUID] | None = None
    installation_id: list[uuid.UUID] | None = None
    modified_id: list[uuid.UUID] | None = None
    deleted_id: list[uuid.UUID] | None = None
    order:  Order | None = None
    type:  Type | None = None
    subtype: str | None = None
    type_timestamp:  TypeTimestamp | None = None
    group_by:  GroupBy | None = None
    time_aggregation:  TimeAggregation | None = None
    operation:  Operation | None = None
    percentile: confloat(ge=0.0, le=1.0) | None = None
    number: bool | None = None
    log_type: str | None = None
    together: bool | None = None
    query_internal_time: bool | None = None



from .wappstorest import WappstoRest
from .wappstows import WappstoWebSocket

from .schemas import WappstoSchema

from .schemas.base import WappstoService
from .schemas.base import WappstoEnv
from .path import WappstoPath

__version__ = "0.0.4"
__auther__ = "Seluxit A/S"

__all__ = [
    'WappstoRest',
    'WappstoWebSocket',
    'WappstoService',
    'WappstoEnv',
    'WappstoSchema',
    'WappstoPath'
]



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:28:52+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Any

from pydantic import BaseModel, Extra, confloat, constr


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class RecurrencyEnum(Enum):
    month = 'month'
    quarter = 'quarter'
    year = 'year'


class LanguageEnum(Enum):
    en = 'en'
    dk = 'dk'


class Information(BaseModel):
    company: str | None = None
    street: str | None = None
    post_number: str | None = None
    city: str | None = None
    country: str | None = None
    tax_code: str | None = None
    tax_code_type: str | None = None
    email: str | None = None
    phone: str | None = None


class TheirReferenceItem(BaseModel):
    name: str | None = None
    email: str | None = None
    phone: str | None = None


class OurReferenceItem(BaseModel):
    name: str | None = None
    email: str | None = None
    phone: str | None = None


class RecurrencyEnum1(Enum):
    one_time = 'one_time'
    subscription = 'subscription'


class Type(Enum):
    network = 'network'
    installation = 'installation'
    application = 'application'
    creator = 'creator'
    generic = 'generic'


class InvoiceItem(BaseModel):
    class Config:
        extra = Extra.allow

    recurrency:  RecurrencyEnum1 | None = None
    type:  Type | None = None
    filter: str | None = None
    filter_by_name: str | None = None
    filter_by_application: uuid.UUID | None = None
    quantity: int | None = None
    cost: int | None = None
    description: str | None = None
    dynamic: bool | None = None
    unit: str | None = None


class CustomerInvoiceItem(BaseModel):
    class Config:
        extra = Extra.allow

    meta:  MetaItem | None = None
    name: str | None = None
    description: str | None = None
    paid: bool | None = None
    sent: bool | None = None
    currency: str | None = None
    archived: bool | None = None
    due_days_after_sending_invoice: int | None = None
    due_timestamp: datetime.datetime | None = None
    paid_timestamp: datetime.datetime | None = None
    sent_timestamp: datetime.datetime | None = None
    due_payment_timestamp: datetime.datetime | None = None
    discount: confloat(ge=0.0, le=100.0) | None = None
    discount_before_tax: bool | None = None
    create_automatic_next_invoice: bool | None = None
    tax: confloat(ge=0.0, le=100.0) | None = None
    tax_included: bool | None = None
    note: list[str] | None = None
    invoice: list[InvoiceItem] | None = None


class Customer(BaseModel):
    class Config:
        extra = Extra.allow

    meta:  MetaItem | None = None
    user_id: str | None = None
    seller: uuid.UUID | None = None
    currency: str | None = None
    due_days_after_sending_invoice: int | None = None
    discount_before_tax: bool | None = None
    tax_included: bool | None = None
    note: list[str] | None = None
    next_invoice: dict[str, Any] | None = None
    last_invoice: dict[str, Any] | None = None
    recurrency:  RecurrencyEnum | None = None
    create_automatic_next_invoice: bool | None = None
    archived: bool | None = None
    language:  LanguageEnum | None = None
    information:  Information | None = None
    their_reference:  TheirReferenceItem | None = None
    our_reference:  OurReferenceItem | None = None
    customer_invoice: list[CustomerInvoiceItem | uuid.UUID] | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:26:58+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Any, Literal

from pydantic import BaseModel, Extra, Field, constr


class Assigned(Enum):
    unassigned = 'unassigned'
    admin = 'admin'


class WarningItem(BaseModel):
    message: str | None = None
    data: dict[str, Any] | None = None
    code: int | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class Object(Enum):
    old = 'old'
    new = 'new'


class Type(Enum):
    array = 'array'
    hash = 'hash'
    number = 'number'
    string = 'string'
    boolean = 'boolean'
    generic = 'generic'


class ComponentItem(BaseModel):
    path: list[str | int]
    object:  Object | None = None
    type:  Type | None = None


class Succes(Enum):
    error = 'error'


class Example(BaseModel):
    class Config:
        extra = Extra.forbid

    old_data: Any | None = None
    new_data: Any
    last_update: datetime.datetime | None = None
    last_if_result: bool | None = None
    success: bool | Succes | None = None
    output: Any | None = None


class LastResult(BaseModel):
    timestamp: datetime.datetime | None = None
    success: bool | None = None
    output: Any | None = None


class TriggerOn(Enum):
    always = 'always'
    only_when_becomes_true = 'only_when_becomes_true'
    on_toggle = 'on_toggle'


class SendAs(Enum):
    email = 'email'
    notification = 'notification'


class Alert(BaseModel):
    class Config:
        extra = Extra.forbid

    meta: Meta = Field(..., title='meta-2.1')
    name: str | None = None
    enabled: bool
    component: list[ComponentItem]
    if_operation: list[str] | None = None
    then_operation: list[str] | None = None
    example:  Example | None = None
    last_result:  LastResult | None = None
    trigger_again_after: int | None = None
    trigger_on: TriggerOn
    parser_id: uuid.UUID | None = None
    send_as: SendAs
    subject: str | None = None
    subject_on_false: str | None = None



import uuid
import logging

from pytest_httpx import HTTPXMock

from wappstorest import WappstoEnv
from wappstorest import WappstoPath
from wappstorest import WappstoRest
from wappstorest import WappstoService


logging.root.level = logging.DEBUG


class TestNetworkRest:
    def test_get(self, httpx_mock: HTTPXMock):
        pass
        # session_uuid = uuid.UUID('2d94624a-0a3d-45c6-88ee-ecbf3ff3a980')

        # httpx_mock.add_response(
        #     method="GET",
        #     url='https://wappsto.com/services/2.1/user/me',
        #     match_headers={
        #         'x-session': str(session_uuid),
        #         "Accept": "application/json",
        #         "Content-type": "application/json",
        #     },
        #     json={
        #         'first_name': 'Tom',
        #         'meta': {
        #             'id': str(uuid.uuid4()),
        #             'type': 'user',
        #         }
        #     },  # The Reply
        # )

        # rest = WappstoRest(
        #     env=WappstoEnv.PROD,
        #     session=session_uuid,
        # )

        # reply = rest.service(WappstoService.USER).service_id('me').get()

        # assert reply.first_name == 'Tom'
        # # assert reply.first_name == 'Tom'



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:37:54+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Any

from pydantic import BaseModel, Extra, conint, constr


class EngineEnum(Enum):
    node = 'node'
    python = 'python'


class ExecutableItem(BaseModel):
    class Config:
        extra = Extra.forbid

    engine:  EngineEnum | None = None
    version: str | None = None


class SupportedFeature(Enum):
    foreground = 'foreground'
    background = 'background'
    widget = 'widget'


class DescriptionItem(BaseModel):
    class Config:
        extra = Extra.forbid

    general: str | None = None
    foreground: str | None = None
    background: str | None = None
    widget: str | None = None
    version: str | None = None


class StatusEnum(Enum):
    idle = 'idle'
    commit = 'commit'
    revision = 'revision'
    publish = 'publish'
    uncommit = 'uncommit'
    unpublish = 'unpublish'
    republish = 'republish'
    disable = 'disable'
    reenable = 'reenable'
    duplicate = 'duplicate'
    pending = 'pending'
    uncommitted = 'uncommitted'
    published = 'published'
    not_updated = 'not updated'
    unpublished = 'unpublished'
    disabled = 'disabled'


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class PermissionItem(BaseModel):
    create: list[str] | None = None
    permit_to_send_email: bool | None = None
    permit_to_send_sms: bool | None = None


class ConcernItem(BaseModel):
    class Config:
        extra = Extra.forbid

    concern: str | None = None
    line: conint(ge=0, multiple_of=1) | None = None


class ConcernFileItem(BaseModel):
    class Config:
        extra = Extra.forbid

    timestamp: datetime.datetime | None = None
    concern: list[ConcernItem] | None = None


class FileItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str | None = None
    type: str | None = None
    size: conint(ge=0, multiple_of=1) | None = None
    concernFile: list[ConcernFileItem] | None = None
    meta:  MetaItem | None = None


class Version(BaseModel):
    class Config:
        extra = Extra.allow

    name: str | None = None
    name_identifier: constr(regex=r'[0-9a-z_]') | None = None
    author: str | None = None
    version_app: constr(regex=r'^(\d+\.\d+\.\d+)$') | None = None
    commit: str | None = None
    date: datetime.datetime | None = None
    title: str | None = None
    session_user: bool | None = None
    concern: str | None = None
    native: bool | None = None
    name_folder: str | None = None
    icon: uuid.UUID | None = None
    version_to_clone: uuid.UUID | None = None
    uninstallable: bool | None = None
    executable:  ExecutableItem | None = None
    max_number_installation: conint(ge=1) | None = None
    object_requested: dict[str, Any] | None = None
    supported_features: list[SupportedFeature] | None = None
    description:  DescriptionItem | None = None
    status:  StatusEnum | None = None
    used_files: dict[str, Any] | None = None
    meta:  MetaItem | None = None
    permission:  PermissionItem | None = None
    file: list[FileItem | uuid.UUID] | None = None



from enum import Enum

from typing import Any
from typing import Literal

from pydantic import BaseModel
from pydantic import Extra
from pydantic import UUID4

from .base import WappstoService


class WappstoVersion(Enum):
    V2_0 = "2.0"
    V2_1 = "2.1"


class ApiMetaTypes(Enum):
    IDLIST = "idlist"
    DELETELIST = "deletelist"
    ATTRIBUTELIST = "attributelist"


class ApiMetaInfo(BaseModel):
    type: ApiMetaTypes  # Merge with MetaAPIData?
    version: WappstoVersion


class IdListMeta(BaseModel):
    type: Literal['idlist']
    version: WappstoVersion


class DeleteListMeta(BaseModel):
    type: Literal['deletelist']
    version: WappstoVersion


class childInfo(BaseModel):
    type: WappstoService
    version: WappstoVersion


class IdList(BaseModel):
    child: list[ApiMetaInfo]
    id: list[UUID4]
    more: bool
    limit: int
    count: int
    meta: IdListMeta


class DeleteList(BaseModel):
    deleted: list[UUID4]
    code: int
    message: str = "Deleted"
    meta: DeleteListMeta

    # TODO: Remove: 'Extra.forbid' and use meta->type to force type.


class AttributeList(BaseModel):
    data: dict[UUID4, Any]
    more: bool
    path: str
    meta: ApiMetaInfo



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:30:47+00:00

from __future__ import annotations

from typing import Any

from pydantic import BaseModel, Extra, Field, constr


class WarningItem(BaseModel):
    message: str | None = None
    data: dict[str, Any] | None = None
    code: int | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.forbid

    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    warning: list[WarningItem] | None = None


class Fetch(BaseModel):
    class Config:
        extra = Extra.forbid

    data: dict[str, Any]
    meta: Meta = Field(..., title='meta_list-2.1')



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:33:38+00:00

from __future__ import annotations

import uuid
from enum import Enum
from typing import Any

from pydantic import BaseModel, Extra, Field, constr


class AccessTokenMethod(Enum):
    get = 'get'
    post = 'post'


class OauthVersion(Enum):
    field_1_0 = '1.0'
    field_1_0a = '1.0a'
    field_2_0 = '2.0'


class RefreshAccessTokenMethod(Enum):
    get = 'get'
    post = 'post'


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class OauthExternal(BaseModel):
    class Config:
        extra = Extra.allow

    name: str
    description: str | None = None
    api_callback: str | None = None
    api_key: str
    api_secret_key: str
    api_site: str
    api_extra_request: dict[str, Any] | None = None
    api_extra_request_required: list[str] | None = None
    api_extra_access_token: dict[str, Any] | None = None
    access_token_method:  AccessTokenMethod | None = None
    application_id: uuid.UUID | None = None
    oauth_version: OauthVersion
    oauth_request_token_url: str | None = None
    oauth_access_token_url: str | None = None
    oauth_refresh_access_token_url: str | None = None
    api_extra_refresh_access_token: dict[str, Any] | None = None
    refresh_access_token_method:  RefreshAccessTokenMethod | None = None
    meta: Meta | None = Field(None, title='meta-2.1')



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:38:08+00:00

from __future__ import annotations

import uuid
from typing import Any
from pydantic import BaseModel, Extra, constr


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class GridProperty1(BaseModel):
    class Config:
        extra = Extra.forbid

    height: float | None = None
    width: float | None = None
    x: float | None = None
    y: float | None = None
    moved: bool | None = None
    static: bool | None = None


class Item(BaseModel):
    class Config:
        extra = Extra.forbid

    type: str | None = None
    name: str | None = None
    id: uuid.UUID | None = None
    children: Any | None = None


class Grid(BaseModel):
    class Config:
        extra = Extra.forbid

    large: Any | None = None
    medium: Any | None = None
    small: Any | None = None


class DashboardItem(BaseModel):
    class Config:
        extra = Extra.forbid

    grid:  Grid | None = None


class Widget(BaseModel):
    class Config:
        extra = Extra.forbid

    meta:  MetaItem | None = None
    name: str | None = None
    type: str | None = None
    configuration: dict[str, Any] | None = None
    items: list[Item] | None = None
    dashboard:  DashboardItem | None = None



from .output.acl import Acl
# from .output.admin_feature import AdminFeature
# from .output.alert import Alert
# from .output.analytics import Analytics
# from .output.analytics_model import AnalyticsModel
# from .output.application import Application
# from .output.application_product import ApplicationProduct
# from .output.azure import Azure
# from .output.bus_stream import BusStream
# from .output.cockroach import Cockroach
# from .output.console import Console
# from .output.couchbase import Couchbase
# from .output.creator import Creator
# from .output.customer import Customer
# from .output.customer_invoice import CustomerInvoice
# from .output.dashboard import Dashboard
# from .output.data import Data
# from .output.email_draft import EmailDraft
# from .output.email_send import EmailSend
# from .output.email_translation import EmailTranslation
# from .output.eventlog import Eventlog
# from .output.expected_state import ExpectedState
# from .output.extsync import Extsync
# from .output.extsync_response import ExtsyncResponse
# from .output.file import File
# from .output.filter import Filter
# from .output.firmware import Firmware
# from .output.fite import Fite
# from .output.homepage import Homepage
# from .output.installation import Installation
# from .output.mda import Mda
# from .output.meta import Meta
# from .output.metrics import Metrics
# from .output.mqtt import Mqtt
# from .output.n1ql import N1ql
# from .output.notification import Notification
# from .output.oauth import Oauth
# from .output.oauth_client import OauthClient
# from .output.oauth_connect import OauthConnect
# from .output.oauth_external import OauthExternal
# from .output.ontology import Ontology
# from .output.parser import Parser
# from .output.permission import Permission
# from .output.permission_group import PermissionGroup
# from .output.permission_rule import PermissionRule
# from .output.point import Point
# from .output.price import Price
# from .output.product import Product
# from .output.promote import Promote
# from .output.prototype import Prototype
# from .output.purse import Purse
# from .output.recovery import Recovery
# from .output.recovery_cleanup import RecoveryCleanup
# from .output.recovery_repeat import RecoveryRepeat
# from .output.repeat import Repeat
# from .output.register import Register
# from .output.seller import Seller
from .output.session import Session
# from .output.sms_draft import SmsDraft
# from .output.sms_send import SmsSend
# # from .output.state import State
# from .output.status import Status
# from .output.stream import Stream
# from .output.subscription import Subscription
# from .output.subuser import Subuser
# from .output.tax import Tax

# from .output.transfer_billing import TransferBilling
# from .output.transfer_translation import TransferTranslation
# from .output.transfer_email import TransferEmail
# from .output.transfer_user import TransferUser

from .output.user import User
# from .output.version import Version
# from .output.virtualuser import Virtualuser
# from .output.websocket import Websocket
# from .output.widget import Widget

from .output.network import Network
from .output.network import Device
from .output.network import Value
from .output.network import State

from .output.idlist import Idlist as IdList
from .output.deletelist import Deletelist as DeleteList
# from .output.attributelist import Attributelist as AttributeList
from .id_list import AttributeList

from .websocket_schema import EventStreamSchema

__all__ = [
    'IdList',
    'DeleteList',
    'AttributeList',
    'EventStreamSchema',

    'Network',
    'Device',
    'Value',
    'State',

    'Acl',
    # 'AdminFeature',
    # 'Alert',
    # 'Analytics',
    # 'AnalyticsModel',
    # 'Application',
    # 'ApplicationProduct',
    # 'Azure',
    # 'BusStream',
    # 'Cockroach',
    # 'Console',
    # 'Couchbase',
    # 'Creator',
    # 'Customer',
    # 'CustomerInvoice',
    # 'Dashboard',
    # 'Data',
    # 'EmailDraft',
    # 'EmailSend',
    # 'EmailTranslation',
    # 'Eventlog',
    # 'ExpectedState',
    # 'Extsync',
    # 'ExtsyncResponse',
    # 'File',
    # 'Filter',
    # 'Firmware',
    # 'Fite',
    # 'Homepage',
    # 'Installation',
    # 'Mda',
    # 'Meta',
    # 'Metrics',
    # 'Mqtt',
    # 'N1ql',
    # 'Notification',
    # 'Oauth',
    # 'OauthClient',
    # 'OauthConnect',
    # 'OauthExternal',
    # 'Ontology',
    # 'Parser',
    # 'Permission',
    # 'PermissionGroup',
    # 'PermissionRule',
    # 'Point',
    # 'Price',
    # 'Product',
    # 'Promote',
    # 'Prototype',
    # 'Purse',
    # 'Recovery',
    # 'RecoveryCleanup',
    # 'Repeat',
    # 'Register',
    # 'RecoveryRepeat',
    # 'Seller',
    'Session',
    # 'SmsDraft',
    # 'SmsSend',
    # 'Status',
    # 'Stream',
    # 'Subscription',
    # 'Subuser',
    # 'Tax',
    # 'TransferBilling',
    # 'TransferTranslation',
    # 'TransferEmail',
    # 'TransferUser',
    'User',
    # 'Version',
    # 'Virtualuser',
    # 'Websocket',
    # 'Widget',
]



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:33:47+00:00

from __future__ import annotations

import uuid
from enum import Enum
from typing import Any

from pydantic import BaseModel, Extra, Field, constr


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Operation(Enum):
    table = 'table'
    message = 'message'


class Message(BaseModel):
    false_message: str | None = None
    true_message: str


class Version(Enum):
    field_2_1 = '2.1'
    field_2_1_1 = '2.1'


class Condition(Enum):
    lessThan = 'lessThan'
    field_ = '<'
    lessOrEqualThan = 'lessOrEqualThan'
    field__ = '<='
    moreThan = 'moreThan'
    field__1 = '>'
    moreOrEqualThan = 'moreOrEqualThan'
    field___1 = '>='
    equal = 'equal'
    field___2 = '=='
    different = 'different'
    field___3 = '!='
    contains = 'contains'
    field__2 = '='
    always = 'always'
    default = 'default'


class Color(BaseModel):
    condition:  Condition | None = None
    value: str | None = None
    conditionValue: str | None = None


class Column(BaseModel):
    data: str | None = None
    title: str | None = None
    colors: list[Color] | None = None


class Table(BaseModel):
    class Config:
        extra = Extra.forbid

    service: str
    version:  Version | None = None
    parser: str | None = None
    columns: list[Column] | None = None


class Example(BaseModel):
    class Config:
        extra = Extra.allow

    input: Any | None = None


class Parser(BaseModel):
    class Config:
        extra = Extra.allow

    meta: Meta | None = Field(None, title='meta-2.1')
    name: str | None = None
    operation: Operation
    message:  Message | None = None
    table:  Table | None = None
    example:  Example | None = None
    to_parse: Any | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:35:31+00:00

from __future__ import annotations

from enum import Enum
from pydantic import BaseModel, Extra, constr


class Request(Enum):
    delete_user = 'delete_user'
    recovery_password = 'recovery_password'
    change_password = 'change_password'
    confirm_new_username = 'confirm_new_username'
    confirm_registration = 'confirm_registration'
    confirm_deletion_user = 'confirm_deletion_user'
    verify_email = 'verify_email'
    confirm_email = 'confirm_email'
    verify_changed_email = 'verify_changed_email'
    confirm_changed_email = 'confirm_changed_email'
    verify_phone_number = 'verify_phone_number'
    confirm_phone_number = 'confirm_phone_number'


class Register(BaseModel):
    class Config:
        extra = Extra.forbid

    key: constr(regex=r'[0-9a-fA-F]{64}|[0-9]{6}') | None = None
    request:  Request | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:32:34+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum

from pydantic import BaseModel, Extra, Field, constr


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Operation(Enum):
    count_online_iot = 'count_online_iot'
    count_online_at_least_once = 'count_online_at_least_once'


class Type(Enum):
    network = 'network'
    device = 'device'
    value = 'value'
    state = 'state'


class Limitation(BaseModel):
    class Config:
        extra = Extra.forbid

    type:  Type | None = None
    attribute: str | None = None
    value: list | None = None


class Resolution(Enum):
    second = 'second'
    minute = 'minute'
    hour = 'hour'
    day = 'day'
    week = 'week'
    month = 'month'


class Query(BaseModel):
    class Config:
        extra = Extra.forbid

    begin: datetime.datetime | None = None
    end: datetime.datetime | None = None
    ids: list[uuid.UUID] | None = None
    limitations: list[Limitation] | None = None
    resolution:  Resolution | None = None


class Metrics(BaseModel):
    class Config:
        extra = Extra.allow

    meta: Meta | None = Field(None, title='meta-2.1')
    operation: Operation
    query:  Query | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:26:59+00:00

from __future__ import annotations

import uuid
from enum import Enum
from typing import Any

from pydantic import BaseModel, Extra, Field, constr


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Object(Enum):
    old = 'old'
    new = 'new'


class Type(Enum):
    array = 'array'
    hash = 'hash'
    number = 'number'
    string = 'string'
    boolean = 'boolean'
    generic = 'generic'


class ComponentItem(BaseModel):
    path: list[str | int]
    object:  Object | None = None
    type:  Type | None = None


class Example(BaseModel):
    class Config:
        extra = Extra.allow

    old_data: Any | None = None
    new_data: Any


class TriggerOn(Enum):
    always = 'always'
    only_when_becomes_true = 'only_when_becomes_true'
    on_toggle = 'on_toggle'


class SendAs(Enum):
    email = 'email'
    notification = 'notification'


class Alert(BaseModel):
    class Config:
        extra = Extra.allow

    meta: Meta | None = Field(None, title='meta-2.1')
    name: str | None = None
    enabled: bool | None = None
    component: list[ComponentItem]
    if_operation: list[str] | None = None
    then_operation: list[str] | None = None
    example:  Example | None = None
    trigger_again_after: int | None = None
    trigger_on:  TriggerOn | None = None
    parser_id: uuid.UUID | None = None
    send_as:  SendAs | None = None
    subject: str | None = None
    subject_on_false: str | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:27:20+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Any

from pydantic import BaseModel, Extra, Field, conint, constr


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class EngineEnum(Enum):
    node = 'node'
    python = 'python'


class ExecutableItem(BaseModel):
    class Config:
        extra = Extra.forbid

    engine:  EngineEnum | None = None
    version: str | None = None


class SupportedFeature(Enum):
    foreground = 'foreground'
    background = 'background'
    widget = 'widget'


class DescriptionItem(BaseModel):
    class Config:
        extra = Extra.forbid

    general: str | None = None
    foreground: str | None = None
    background: str | None = None
    widget: str | None = None
    version: str | None = None


class StatusEnum(Enum):
    idle = 'idle'
    commit = 'commit'
    revision = 'revision'
    publish = 'publish'
    uncommit = 'uncommit'
    unpublish = 'unpublish'
    republish = 'republish'
    disable = 'disable'
    reenable = 'reenable'
    duplicate = 'duplicate'
    pending = 'pending'
    uncommitted = 'uncommitted'
    published = 'published'
    not_updated = 'not updated'
    unpublished = 'unpublished'
    disabled = 'disabled'


class PermissionItem(BaseModel):
    create: list[str] | None = None
    permit_to_send_email: bool | None = None
    permit_to_send_sms: bool | None = None


class ConcernItem(BaseModel):
    class Config:
        extra = Extra.forbid

    concern: str | None = None
    line: conint(ge=0, multiple_of=1) | None = None


class ConcernFileItem(BaseModel):
    class Config:
        extra = Extra.forbid

    timestamp: datetime.datetime | None = None
    concern: list[ConcernItem] | None = None


class FileItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str | None = None
    type: str | None = None
    size: conint(ge=0, multiple_of=1) | None = None
    concernFile: list[ConcernFileItem] | None = None
    meta:  MetaItem | None = None


class VersionItem(BaseModel):
    class Config:
        extra = Extra.allow

    name: str | None = None
    name_identifier: constr(regex=r'[0-9a-z_]') | None = None
    author: str | None = None
    version_app: constr(regex=r'^(\d+\.\d+\.\d+)$') | None = None
    commit: str | None = None
    date: datetime.datetime | None = None
    title: str | None = None
    session_user: bool | None = None
    concern: str | None = None
    native: bool | None = None
    name_folder: str | None = None
    icon: uuid.UUID | None = None
    version_to_clone: uuid.UUID | None = None
    uninstallable: bool | None = None
    executable:  ExecutableItem | None = None
    max_number_installation: conint(ge=1) | None = None
    object_requested: dict[str, Any] | None = None
    supported_features: list[SupportedFeature] | None = None
    description:  DescriptionItem | None = None
    status:  StatusEnum | None = None
    used_files: dict[str, Any] | None = None
    meta:  MetaItem | None = None
    permission:  PermissionItem | None = None
    file: list[FileItem | uuid.UUID] | None = None


class ExternalItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str | None = None
    api_key: str | None = None
    api_secret_key: str | None = None
    api_site: str | None = None


class OauthClientItem(BaseModel):
    class Config:
        extra = Extra.allow

    name: str | None = None
    company: str | None = None
    description: str | None = None
    homepage_url: str | None = None
    expires: conint(ge=0) | None = None
    client_id: uuid.UUID | None = None
    client_secret: str | None = None
    redirect_uri: list[str] | None = None
    path_access_token: list[str] | None = None
    meta:  MetaItem | None = None
    external: list[ExternalItem] | None = None


class AccessTokenMethodEnum(Enum):
    get = 'get'
    post = 'post'


class OauthVersionEnum(Enum):
    field_1_0 = '1.0'
    field_1_0a = '1.0a'
    field_2_0 = '2.0'


class RefreshAccessTokenMethodEnum(Enum):
    get = 'get'
    post = 'post'


class OauthExternalItem(BaseModel):
    class Config:
        extra = Extra.allow

    name: str | None = None
    description: str | None = None
    api_callback: str | None = None
    api_key: str | None = None
    api_secret_key: str | None = None
    api_site: str | None = None
    api_extra_request: dict[str, Any] | None = None
    api_extra_request_required: list[str] | None = None
    api_extra_access_token: dict[str, Any] | None = None
    access_token_method:  AccessTokenMethodEnum | None = None
    application_id: uuid.UUID | None = None
    oauth_version:  OauthVersionEnum | None = None
    oauth_request_token_url: str | None = None
    oauth_access_token_url: str | None = None
    oauth_refresh_access_token_url: str | None = None
    api_extra_refresh_access_token: dict[str, Any] | None = None
    refresh_access_token_method:  RefreshAccessTokenMethodEnum | None = None
    meta:  MetaItem | None = None


class TypeEnum(Enum):
    one_time = 'one_time'
    subscription = 'subscription'


class IntervalEnum(Enum):
    day = 'day'
    week = 'week'
    month = 'month'
    year = 'year'


class RecurringItem(BaseModel):
    class Config:
        extra = Extra.forbid

    interval:  IntervalEnum | None = None
    interval_count: conint(ge=1) | None = None


class TaxItem(BaseModel):
    id: uuid.UUID | None = None


class ApplicationProductItem(BaseModel):
    class Config:
        extra = Extra.allow

    meta:  MetaItem | None = None
    name: str | None = None
    description: str | None = None
    active: bool | None = None
    type:  TypeEnum | None = None
    currency: str | None = None
    amount: conint(ge=1) | None = None
    recurring:  RecurringItem | None = None
    tax:  TaxItem | None = None
    trial_period_days: conint(ge=1) | None = None


class Application(BaseModel):
    class Config:
        extra = Extra.allow

    name: str | None = None
    name_identifier: constr(regex=r'[0-9a-z_]') | None = None
    meta:  MetaItem | None = None
    permit_free: bool | None = None
    version: list[VersionItem | uuid.UUID] | None = None
    oauth_client: list[OauthClientItem | uuid.UUID] | None = Field(None, max_items=1)
    oauth_external: list[OauthExternalItem | uuid.UUID] | None = None
    application_product: list[ApplicationProductItem | uuid.UUID] | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:27:16+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Any, Literal

from pydantic import BaseModel, Extra, Field, conint, constr


class SupportedFeature(Enum):
    foreground = 'foreground'
    background = 'background'
    widget = 'widget'


class Description(BaseModel):
    class Config:
        extra = Extra.forbid

    general: str | None = None
    foreground: str | None = None
    background: str | None = None
    widget: str | None = None
    version: str | None = None


class PublishedVersion(BaseModel):
    class Config:
        extra = Extra.forbid

    version_app: str | None = None
    author: str | None = None
    session_user: bool | None = None
    native: bool | None = None
    uninstallable: bool | None = None
    max_number_installation: conint(ge=1) | None = None
    supported_features: list[SupportedFeature] | None = None
    description: Description | None = None


class Installed(BaseModel):
    last_update: datetime.datetime | None = None
    quantity: int | None = None


class Assigned(Enum):
    unassigned = 'unassigned'
    admin = 'admin'


class WarningItem(BaseModel):
    message: str | None = None
    data: dict[str, Any] | None = None
    code: int | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class Engine(Enum):
    node = 'node'
    python = 'python'


class Executable(BaseModel):
    class Config:
        extra = Extra.forbid

    engine: Engine | None = None
    version: str | None = None


class Status(Enum):
    idle = 'idle'
    commit = 'commit'
    revision = 'revision'
    publish = 'publish'
    uncommit = 'uncommit'
    unpublish = 'unpublish'
    republish = 'republish'
    disable = 'disable'
    reenable = 'reenable'
    duplicate = 'duplicate'
    pending = 'pending'
    uncommitted = 'uncommitted'
    published = 'published'
    not_updated = 'not updated'
    unpublished = 'unpublished'
    disabled = 'disabled'


class Meta1(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None
    last_committed: uuid.UUID | None = None
    last_published: uuid.UUID | None = None


class Permission(BaseModel):
    create: list[str] | None = None
    permit_to_send_email: bool | None = None
    permit_to_send_sms: bool | None = None


class ConcernItem(BaseModel):
    class Config:
        extra = Extra.forbid

    concern: str
    line: conint(ge=0, multiple_of=1)


class ConcernFileItem(BaseModel):
    class Config:
        extra = Extra.forbid

    timestamp: datetime.datetime
    concern: list[ConcernItem] = Field(..., min_items=1)


class Meta2(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class FileItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str | None = None
    type: str | None = None
    size: conint(ge=0, multiple_of=1) | None = None
    published: bool | None = None
    concernFile: list[ConcernFileItem] | None = None
    meta: Meta2 = Field(..., title='meta-2.1')


class VersionItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    name_identifier: constr(regex=r'[0-9a-z_]') | None = None
    application: uuid.UUID | None = None
    author: str | None = None
    version_app: constr(regex=r'^(\d+\.\d+\.\d+)$') | None = None
    commit: str | None = None
    title: str | None = None
    session_user: bool | None = None
    concern: str | None = None
    native: bool | None = None
    name_folder: str | None = None
    icon: uuid.UUID | None = None
    priority: conint(ge=1, multiple_of=1) | None = None
    time_commit: datetime.datetime | None = None
    version_to_clone: uuid.UUID | None = None
    installation: list[uuid.UUID] | None = None
    uninstallable: bool | None = None
    published: bool | None = None
    executable: Executable | None = None
    max_number_installation: conint(ge=1, multiple_of=1) | None = None
    object_requested: dict[str, Any] | None = None
    supported_features: list[SupportedFeature] | None = None
    description: Description | None = None
    status: Status
    used_files: dict[str, Any]
    installed: Installed | None = None
    meta: Meta1 = Field(..., title='meta-2.1')
    permission: Permission | None = None
    file: list[FileItem | uuid.UUID] | None = None


class Meta3(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class ExternalItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    api_key: str
    api_secret_key: str
    api_site: str


class OauthClientItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    company: str | None = None
    description: str | None = None
    homepage_url: str | None = None
    expires: conint(ge=0) | None = None
    client_id: uuid.UUID
    client_secret: str | None = None
    redirect_uri: list[str] | None = None
    path_access_token: list[str] | None = None
    application_id: uuid.UUID
    installation_id: list[uuid.UUID]
    meta: Meta3 = Field(..., title='meta-2.1')
    external: list[ExternalItem] | None = None


class AccessTokenMethod(Enum):
    get = 'get'
    post = 'post'


class OauthVersion(Enum):
    field_1_0 = '1.0'
    field_1_0a = '1.0a'
    field_2_0 = '2.0'


class RefreshAccessTokenMethod(Enum):
    get = 'get'
    post = 'post'


class Meta4(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class OauthExternalItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    description: str | None = None
    api_callback: str | None = None
    api_key: str
    api_secret_key: str | None = None
    api_site: str
    api_extra_request: dict[str, Any] | None = None
    api_extra_request_required: list[str] | None = None
    api_extra_access_token: dict[str, Any] | None = None
    access_token_method:  AccessTokenMethod | None = None
    application_id: uuid.UUID | None = None
    oauth_version: OauthVersion
    oauth_request_token_url: str | None = None
    oauth_access_token_url: str | None = None
    oauth_refresh_access_token_url: str | None = None
    api_extra_refresh_access_token: dict[str, Any] | None = None
    refresh_access_token_method:  RefreshAccessTokenMethod | None = None
    meta: Meta4 = Field(..., title='meta-2.1')


class Meta5(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class Type(Enum):
    one_time = 'one_time'
    subscription = 'subscription'


class Interval(Enum):
    day = 'day'
    week = 'week'
    month = 'month'
    year = 'year'


class Recurring(BaseModel):
    class Config:
        extra = Extra.forbid

    interval: Interval
    interval_count: conint(ge=1)


class PricesWithTax(BaseModel):
    amount: int | None = None
    prepaid_extra_cost: int | None = None
    topup_extra_cost: int | None = None


class PricesWithoutTax(BaseModel):
    amount: int | None = None
    prepaid_extra_cost: int | None = None
    topup_extra_cost: int | None = None


class Tax(BaseModel):
    id: uuid.UUID | None = None
    prices_with_tax: PricesWithTax | None = None
    prices_without_tax: PricesWithoutTax | None = None
    name: str | None = None
    description: str | None = None
    jurisdiction: str | None = None
    inclusive: bool | None = None
    percentage: int | None = None
    state: str | None = None


class ApplicationProductItem(BaseModel):
    class Config:
        extra = Extra.forbid

    meta: Meta5 | None = Field(None, title='meta-2.1')
    name: str | None = None
    description: str | None = None
    active: bool | None = None
    deleted: bool | None = None
    stripe: dict[str, Any] | None = None
    type:  Type | None = None
    currency: str | None = None
    amount: conint(ge=1) | None = None
    recurring: Recurring | None = None
    tax: Tax | None = None
    trial_period_days: conint(ge=1) | None = None


class Application(BaseModel):
    class Config:
        extra = Extra.forbid

    last_published: uuid.UUID | None = None
    last_committed: uuid.UUID | None = None
    name: str
    installation: list[uuid.UUID] | None = None
    published_version:  PublishedVersion | None = None
    name_identifier: constr(regex=r'[0-9a-z_]') | None = None
    published: bool | None = None
    permit_free: bool | None = None
    installed: Installed | None = None
    meta: Meta = Field(..., title='meta-2.1')
    version: list[VersionItem | uuid.UUID]
    oauth_client: list[OauthClientItem | uuid.UUID] = Field(..., max_items=1)
    oauth_external: list[OauthExternalItem | uuid.UUID]
    application_product: list[ApplicationProductItem | uuid.UUID]



# -*- coding: utf-8 -*-
"""The Wappsto Rest Python API module."""
from __future__ import annotations

import uuid
import logging

from typing import Literal
from typing import overload

from .wappsto_service_builder import WappstoServiceBuilder

from .wappsto_service_builder import WappstoSessionBuilder
from .wappsto_service_builder import WappstoUserBuilder
from .wappsto_service_builder import get_builder

from .schemas.base import WappstoEnv
from .schemas.base import WappstoService
from .schemas.base import WappstoVersion
from .schemas import WappstoSchema

from rich.console import Console

console = Console()

name = "WappstoRest"
__version__ = "v0.0.1"


class WappstoRest:
    """
    WappstoRest is a helper class for interacting with the Wappsto REST service.

    WappstoRest offers an Fluent interface to build up the request make to Wappsto.
    """

    def __init__(
        self,
        env: WappstoEnv = WappstoEnv.PROD,
        version: WappstoVersion = WappstoVersion.V2_1,
        session: uuid.UUID | None = None,
        username: str | None = None,
        password: str | None = None,
    ):
        """
        Initialization of the Wappsto REST connection.

        Args:
            env: The Wappsto Environment to connect to. (Default: PROD)
            session: (Optional) If a user session created to be used.
            username: (Optional) The Wappsto Login, to create a user session.
            password: (Optional) Password for the Wappsto Login.
        """
        self.log = logging.getLogger(__name__)
        self.log.addHandler(logging.NullHandler())

        self.vr: WappstoVersion = version
        self.env: WappstoEnv = env
        self.admin_session: uuid.UUID | None = None
        self.default_session: uuid.UUID | None = None

        self.base_url = "wappsto.com"
        self.service_url = f"{self.base_url}/services"
        self.__set_wappsto_enviroment(env=env)
        if session:
            self.set_session(session=session)

    def __set_wappsto_enviroment(self, env: WappstoEnv):
        self.base_url = f"{env.value + '.' if env != WappstoEnv.PROD else ''}wappsto.com"
        self.service_url = f"{self.base_url}/services"

    def set_session(self, session: uuid.UUID, admin: bool = False):
        """
        Set a premake session UUID.

        Args:
            session: The Session UUID to be set.
            admin: If True, the session UUID given will be
                   save as a admin session.
        """
        if admin is True:
            self.admin_session = session
        else:
            self.default_session = session

    def login(self, username: str, password: str, admin: bool = False) -> bool:
        """
        Login and create a session for given user.

        # TODO: Add handle of admin.

        Args:
            username: The Wappsto Login.
            password: Password for the Wappsto Login.
            admin: If True, login as a admin.

        Returns:
            True, if the task was successful else
            False.
        """
        data: WappstoSchema.Create.Session = WappstoSchema.Create.Session(
            username=username,
            password=password,
            remember_me=False
        )

        session = self.service(WappstoService.SESSION)

        if admin is True:
            session = session.admin()

        new_session = session.create(
            data=data
        )

        self.default_session = new_session.meta.id

        return True

    def logout(self, admin: bool = False) -> bool:
        """
        Logout of the session.

        # TODO: Add handle of admin.

        Args:
            admin: IF True, logout the admin session.

        Returns:
            True, if the task was successful else
            False.
        """
        if self.default_session is None:
            return False

        self.service(WappstoService.SESSION).service_id(
            self.default_session,
        ).delete()

        self.default_session = None

        return True

    @overload
    def service(
        self,
        wappsto_service: Literal[WappstoService.SESSION.value]
    ) -> WappstoSessionBuilder: ...

    @overload
    def service(
        self,
        wappsto_service: Literal[WappstoService.USER.value]
    ) -> WappstoUserBuilder: ...

    def service(self, wappsto_service: WappstoService) -> WappstoServiceBuilder:
        """
        Start building an service request.

        Args:
            wappsto_service: The Wappsto Service that should connect to.

        Returns:
            Reference to self.
        """
        return get_builder.get(wappsto_service, WappstoServiceBuilder)(
            wappsto_service=wappsto_service,
            rest_ref=self
        )



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:26:39+00:00

from __future__ import annotations

import uuid
from pydantic import BaseModel, Extra, conint


class Acl(BaseModel):
    class Config:
        extra = Extra.forbid

    id: list[uuid.UUID] | None = None
    expand: conint(ge=0, multiple_of=1) | None = None
    propagate: bool | None = None
    verbose: bool | list[str] | None = None
    prev_notification: uuid.UUID | None = None



from ..path_helpers import Wrapper


class Application:
    __path__: str = 'acl.permissionitem.restrictionitem.application'
    overrite_list = [
        'sharing',
        'permitted',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Method:
    __path__: str = 'acl.permissionitem.restrictionitem.method'
    overrite_list = [
        'create',
        'update',
        'retrieve',
        'delete',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class State:
    __path__: str = 'acl.permissionitem.restrictionitem.state'
    overrite_list = [
        'min_state',
        'max_state',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Installation:
    __path__: str = 'acl.permissionitem.restrictionitem.installation'
    overrite_list = [
        'shareable'
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class File:
    __path__: str = 'acl.permissionitem.restrictionitem.file'
    overrite_list = [
        'background'
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class RestrictionItem:
    __path__: str = 'acl.permissionitem.restrictionitem'
    overrite_list = [
        'child',
        'create',
        'time',
        'acl_attributes',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    application = Application
    file = File
    installation = Installation
    method = Method
    state = State


class Meta:
    __path__: str = 'acl.permissionitem.meta'
    overrite_list = [
        'id',
        'trace',
        'redirect',
        'icon',
        'tag',
        'tag_by_user',
        'name_by_user',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class PermissionItem:
    __path__: str = 'acl.permissionitem'
    overrite_list = [
        'name',
        'message',
        'propagate',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    restriction = RestrictionItem
    meta = Meta


class Meta1:
    __path__: str = 'acl.meta'
    overrite_list = [
        'id',
        'trace',
        'redirect',
        'icon',
        'tag',
        'tag_by_user',
        'name_by_user',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Acl:
    __path__: str = 'acl'
    overrite_list = [
        'owner',
        'manufacturer',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    permission = PermissionItem
    meta = Meta1



from ..path_helpers import Wrapper


class WarningItem:
    __path__ = 'session.meta.warning'
    overrite_list = [
        'message',
        'data',
        'code',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Meta:
    __path__ = 'session.meta'
    overrite_list = [
        'id',
        'type',
        'version',
        'manufacturer',
        'owner',
        'created',
        'updated',
        'changed',
        'application',
        'revision',
        'trace',
        'oem',
        'deprecated',
        'redirect',
        'size',
        'path',
        'parent',
        'error',
        'icon',
        'tag',
        'tag_by_user',
        'name_by_user',
        'parent_name',
        'read_only',
        'original_id',
        'alert',
        'contract',
    ]

    warning = WarningItem


class Session:
    __path__ = 'session'
    overrite_list = [
        'username',
        'remember_me',
        'system',
        'provider',
        'type',
        'installation',
        'admin',
        'dashboard',
        'test_mode',
        'theme',
        'user',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    meta = Meta



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:28:42+00:00

from __future__ import annotations

import datetime
from enum import Enum

from pydantic import BaseModel, Extra


class Type(Enum):
    user = 'user'
    network = 'network'
    number_networks = 'number_networks'
    mda = 'mda'
    sms = 'sms'


class Counter(BaseModel):
    class Config:
        extra = Extra.forbid

    start: datetime.datetime | None = None
    end: datetime.datetime | None = None
    csv: bool | None = None
    type:  Type | None = None
    subtype: str | None = None
    user_id: list[str] | None = None
    network_id: list[str] | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:32:05+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Any, Literal

from pydantic import BaseModel, Extra, Field, constr


class Background(Enum):
    no_background = 'no_background'
    pending = 'pending'
    not_paid = 'not_paid'
    running = 'running'
    crashed = 'crashed'
    restarting = 'restarting'
    error = 'error'
    failed = 'failed'
    stopped = 'stopped'
    completed = 'completed'
    no_more_points = 'no_more_points'
    not_started = 'not_started'


class Version(Enum):
    uninstalled = 'uninstalled'
    not_updated = 'not updated'
    disabled = 'disabled'
    updated = 'updated'
    editor = 'editor'


class Payment(Enum):
    free = 'free'
    owned = 'owned'
    pending = 'pending'
    paid = 'paid'
    not_paid = 'not_paid'


class Status(BaseModel):
    class Config:
        extra = Extra.forbid

    background:  Background | None = None
    version:  Version | None = None
    payment:  Payment | None = None


class Description(BaseModel):
    class Config:
        extra = Extra.forbid

    general: str | None = None
    foreground: str | None = None
    background: str | None = None
    widget: str | None = None
    version: str | None = None


class Payment1(BaseModel):
    free: bool | None = None
    status: str | None = None
    application_product_id: uuid.UUID | None = None
    created: datetime.datetime | None = None
    current_period_end: datetime.datetime | None = None
    current_period_start: datetime.datetime | None = None
    pending: dict[str, Any] | None = None


class Assigned(Enum):
    unassigned = 'unassigned'
    admin = 'admin'


class WarningItem(BaseModel):
    message: str | None = None
    data: dict[str, Any] | None = None
    code: int | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class Meta1(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class OauthItem(BaseModel):
    class Config:
        extra = Extra.forbid

    client_id: uuid.UUID
    name: str | None = None
    expires: bool | None = None
    expired: bool
    active: bool
    installation_id: uuid.UUID
    meta: Meta1 = Field(..., title='meta-2.1')


class Meta2(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class OauthConnectItem(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str | None = None
    api: uuid.UUID
    installation: uuid.UUID
    token: str
    secret_token: str | None = None
    params: dict[str, Any] | None = None
    meta: Meta2 = Field(..., title='meta-2.1')
    access_token_creation: dict[str, Any] | None = None


class Installation(BaseModel):
    class Config:
        extra = Extra.forbid

    application: uuid.UUID
    version_id: uuid.UUID
    title: str | None = None
    name: str | None = None
    author: str | None = None
    supported_features: list[str]
    version_app: constr(regex=r'\d+(\.\d+)*') | None = None
    session_user: bool
    native: bool | None = None
    name_folder: str | None = None
    icon: uuid.UUID | None = None
    uninstallable: bool | None = None
    upgrade: bool | None = None
    ignore_this_notification: list[str] | None = None
    token_installation: str
    extsync: bool
    permit_to_send_email: bool | None = None
    permit_to_send_sms: bool | None = None
    no_start_background: bool | None = None
    session: uuid.UUID | None = None
    status: Status
    description: Description | None = None
    payment: Payment1
    meta: Meta = Field(..., title='meta-2.1')
    oauth: list[OauthItem | uuid.UUID]
    oauth_connect: list[OauthConnectItem | uuid.UUID]



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:34:07+00:00

from __future__ import annotations

import uuid
import datetime
from enum import Enum
from typing import Any, Literal

from pydantic import BaseModel, Extra, Field, conint, constr


class Assigned(Enum):
    unassigned = 'unassigned'
    admin = 'admin'


class WarningItem(BaseModel):
    message: str | None = None
    data: dict[str, Any] | None = None
    code: int | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.forbid

    id: uuid.UUID | None = None
    type: str | None = None
    version: constr(regex=r'(\d+)\.(\d+)') | None = None
    manufacturer: uuid.UUID | Assigned | None = None
    owner: uuid.UUID | Assigned | None = None
    created: datetime.datetime | Literal['not_available'] | None = None
    updated: datetime.datetime | Literal['not_available'] | None = None
    changed: datetime.datetime | Literal['not_available'] | None = None
    application: uuid.UUID | None = None
    revision: int | None = None
    trace: str | None = None
    oem: str | None = None
    deprecated: bool | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    size: int | None = None
    path: str | None = None
    parent: uuid.UUID | None = None
    error: dict[str, Any] | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None
    warning: list[WarningItem] | None = None
    parent_name: dict[str, Any] | None = None
    read_only: bool | None = None
    original_id: uuid.UUID | None = None
    alert: list[uuid.UUID] | None = None
    contract: list[uuid.UUID] | None = None


class Actual(BaseModel):
    last_update: datetime.datetime | None = None
    point: int | None = None
    document: int | None = None
    log_row: int | None = None
    traffic_in: int | None = None
    traffic_out: int | None = None
    iot_traffic_in: int | None = None
    iot_traffic_out: int | None = None
    stream_traffic: int | None = None
    file: int | None = None
    request: int | None = None
    request_time: int | None = None
    sms: int | None = None


class ActualNetwork(BaseModel):
    last_update: datetime.datetime | None = None
    manufactured_and_owned: int | None = None
    manufactured_not_owned: int | None = None
    owned_not_manufactured: int | None = None
    shared: int | None = None
    total: int | None = None


class CurrentPeriodEndEnum(Enum):
    never = 'never'


class UsageDaily(BaseModel):
    last_update: datetime.datetime | None = None
    point: int | None = None
    document: int | None = None
    log_row: int | None = None
    traffic: int | None = None
    iot_traffic: int | None = None
    stream_traffic: int | None = None
    file: int | None = None
    request: int | None = None
    request_time: int | None = None
    sms: int | None = None


class UserDailyLimit(BaseModel):
    class Config:
        extra = Extra.forbid

    point: conint(ge=0) | Literal['dynamic'] | None = None
    document: conint(ge=0) | None = None
    log_row: conint(ge=0) | None = None
    traffic: conint(ge=0) | None = None
    iot_traffic: conint(ge=0) | None = None
    stream_traffic: conint(ge=0) | None = None
    file: conint(ge=0) | None = None
    request: conint(ge=0) | None = None
    request_time: conint(ge=0) | None = None
    sms: conint(ge=0) | None = None


class NetworkDailyLimit(BaseModel):
    class Config:
        extra = Extra.forbid

    point: conint(ge=0) | Literal['dynamic'] | None = None
    iot_traffic: conint(ge=0) | None = None
    request: conint(ge=0) | None = None
    request_time: conint(ge=0) | None = None


class Point(BaseModel):
    class Config:
        extra = Extra.forbid

    meta: Meta = Field(..., title='meta-2.1')
    actual: Actual
    actual_network: ActualNetwork
    unlimited_period: bool
    unlimited_point: bool
    unlimited_network: bool
    unlimited_sms: bool | None = None
    base_point:  conint(ge=0) | Literal['unlimited']
    extra_point: conint(ge=0) | None = None
    base_sms: conint(ge=0) | Literal['unlimited'] | None = None
    extra_sms: conint(ge=0) | None = None
    base_network: conint(ge=0) | None = None
    current_period_start: datetime.datetime
    current_period_end: datetime.datetime | CurrentPeriodEndEnum
    remaining_point: int | None = None
    remaining_network: int | None = None
    remaining_sms: int | None = None
    remaining_percentage: float
    remaining_network_percentage: float
    remaining_sms_percentage: float | None = None
    are_still_point_available: bool
    are_still_network_available: bool
    are_still_sms_available: bool | None = None
    user_daily_limit: UserDailyLimit | None = None
    network_daily_limit: NetworkDailyLimit | None = None
    usage_daily: UsageDaily | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:35:47+00:00

from __future__ import annotations

from enum import Enum
from typing import Any

from pydantic import BaseModel


class Jsonrpc(Enum):
    field_2_1 = '2.1'


class RpcRequest(BaseModel):
    jsonrpc: Jsonrpc
    method: str
    id: str | float | None = None
    params: list[Any] | dict[str, Any] | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:36:16+00:00

from __future__ import annotations

import uuid
from pydantic import BaseModel, Extra, constr


class Credential(BaseModel):
    class Config:
        extra = Extra.forbid

    access_token: str | None = None
    id_token: str | None = None
    secret: str | None = None


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Session(BaseModel):
    class Config:
        extra = Extra.allow

    username: str | None = None
    password: str | None = None
    remember_me: bool | None = None
    system: str | None = None
    captcha: str | None = None
    provider: str | None = None
    installation: uuid.UUID | None = None
    firebase_token: str | None = None
    credentials:  Credential | None = None
    meta:  MetaItem | None = None
    admin: bool | None = None
    dashboard: str | None = None
    test_mode: bool | None = None
    theme: str | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:35:30+00:00

from __future__ import annotations

import uuid
from pydantic import BaseModel, Extra, constr


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class UserItem(BaseModel):
    class Config:
        extra = Extra.forbid

    first_name: str | None = None
    last_name: str | None = None
    phone: str | None = None
    role: str | None = None
    nickname: str | None = None


class Register(BaseModel):
    class Config:
        extra = Extra.forbid

    username: constr(regex=r'^\S+@\S+\.\S+$') | None = None
    new_username: constr(regex=r'^\S+@\S+\.\S+$') | None = None
    password: str | None = None
    new_password: str | None = None
    captcha: str | None = None
    language: str | None = None
    meta:  MetaItem | None = None
    theme: str | None = None
    firebase_token: str | None = None
    user:  UserItem | None = None





class Wrapper:
    __path__: str

    def __init__(self, name, parent):
        self.__path__ = 'this_' + '.'.join(
            f"{parent}.{name}".split('.')[1:]
        )

    def value_converter(self, value) -> str:
        """
        TODO: Check for datetime, Enum and the likes & costume convert.
        """
        return str(value)

    def __lt__(self, other) -> str:
        """
        less then
        """
        return (
            f"{self.__path__}"
            f"<"
            f"{self.value_converter(other)}"
        )

    def __le__(self, other) -> str:
        """
        less then equel to
        """
        return (
            f"{self.__path__}"
            f"<="
            f"{self.value_converter(other)}"
        )

    def __eq__(self, other) -> str:
        """
        equel to
        """
        return (
            f"{self.__path__}"
            f"=="
            f"{self.value_converter(other)}"
        )

    def __ne__(self, other) -> str:
        """not equel to"""
        return (
            f"{self.__path__}"
            f"!="
            f"{self.value_converter(other)}"
        )

    def __gt__(self, other) -> str:
        """
        greater then
        """
        return (
            f"{self.__path__}"
            f">"
            f"{self.value_converter(other)}"
        )

    def __ge__(self, other) -> str:
        """
        greater then equel to
        """
        return (
            f"{self.__path__}"
            f">="
            f"{self.value_converter(other)}"
        )

    def __invert__(self) -> str:
        """
        """
        return (
            f"~"
            f"{self.__path__}"
        )

    # def __getattr__(self, name):
    #     # https://stackoverflow.com/questions/14512620/is-there-any-way-to-override-the-double-underscore-magic-methods-of-arbitrary
    #     return getattr(self.wrapped, name)



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:33:21+00:00

from __future__ import annotations

import uuid

from enum import Enum
from typing import Any
from pydantic import BaseModel, Extra, Field, constr


class Read(Enum):
    read = 'read'
    unread = 'unread'
    accepted = 'accepted'
    denied = 'denied'
    deleted = 'deleted'


class Base(BaseModel):
    class Config:
        extra = Extra.allow

    to: uuid.UUID | None = None
    ids: list[uuid.UUID] | None = None


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Notification(BaseModel):
    class Config:
        extra = Extra.allow

    read:  Read | None = None
    custom: dict[str, Any] | None = None
    base:  Base | None = None
    meta: Meta | None = Field(None, title='meta-2.1')



from ..path_helpers import Wrapper


class ValueWarningItem:
    __path__ = 'value.meta.warning'
    overrite_list = [
        'message',
        'data',
        'code',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class StateWarningItem:
    __path__ = 'value.state.meta.warning'
    overrite_list = [
        'message',
        'data',
        'code',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class EventlogWarningItem:
    __path__ = 'value.eventlog.meta.warning'
    overrite_list = [
        'message',
        'data',
        'code',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class EventlogMeta:
    __path__ = 'value.eventlog.meta'
    overrite_list = [
        'type',
        'id',
        'version',
        'manufacturer',
        'owner',
        'parent',
        'created',
        'updated',
        'changed',
        'application',
        'deprecated',
        'iot',
        'revision',
        'size',
        'path',
        'oem',
        'accept_manufacturer_as_owner',
        'redirect',
        'error',
        'trace',
        'set',
        'contract',
        'historical',
        'deletion',
        'icon',
        'alert',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    warning = EventlogWarningItem


class ValueMeta:
    __path__ = 'value.meta'
    overrite_list = [
        'type',
        'id',
        'version',
        'manufacturer',
        'owner',
        'parent',
        'created',
        'updated',
        'changed',
        'application',
        'deprecated',
        'iot',
        'revision',
        'size',
        'path',
        'oem',
        'accept_manufacturer_as_owner',
        'redirect',
        'error',
        'trace',
        'set',
        'contract',
        'historical',
        'deletion',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    warning = ValueWarningItem


class Info:
    __path__ = 'value.info'
    overrite_list = [
        'enabled',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class StateMeta:
    __path__ = 'value.state.meta'
    overrite_list = [
        'type',
        'id',
        'version',
        'manufacturer',
        'owner',
        'parent',
        'created',
        'updated',
        'changed',
        'application',
        'deprecated',
        'iot',
        'revision',
        'size',
        'path',
        'oem',
        'accept_manufacturer_as_owner',
        'redirect',
        'error',
        'trace',
        'set',
        'contract',
        'historical',
        'deletion',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    warning = StateWarningItem


class State:
    __path__ = 'value.state'
    overrite_list = [
        'timestamp',
        'data',
        'type',
        'status',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    meta = StateMeta


class ValueEventlogItem:
    __path__ = 'value.eventlog'
    overrite_list = [
        'message',
        'timestamp',
        'info',
        'level',
        'type',
    ]

    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    meta = EventlogMeta


class Number:
    __path__ = 'value.number'
    overrite_list = [
        'min',
        'max',
        'step',
        'mapping',
        'meaningful_zero',
        'ordered_mapping',
        'si_conversion',
        'unit',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class String:
    __path__ = 'value.string'
    overrite_list = [
        'max',
        'encoding',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Blob:
    __path__ = 'value.blob'
    overrite_list = [
        'max',
        'encoding',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Xml:
    __path__ = 'value.xml'
    overrite_list = [
        'xsd',
        'namespace',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )


class Value:
    __path__ = 'value'
    overrite_list = [
        'name',
        'type',
        'description',
        'period',
        'delta',
        'permission',
        'status',
    ]
    for tmpop8yroywhfgkljsdyekj_name in overrite_list:
        locals()[tmpop8yroywhfgkljsdyekj_name] = Wrapper(
            tmpop8yroywhfgkljsdyekj_name, __path__
        )

    meta = ValueMeta
    state = State
    eventlog = EventlogItem
    info = Info
    string = String
    number = Number
    blob = Blob
    xml = Xml



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:29:06+00:00

from __future__ import annotations

import uuid
from pydantic import BaseModel, Extra, constr


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Data(BaseModel):
    meta:  MetaItem | None = None



import uuid
import logging

from pytest_httpx import HTTPXMock

from wappstorest import WappstoEnv
from wappstorest import WappstoPath
from wappstorest import WappstoRest
from wappstorest import WappstoService


logging.root.level = logging.DEBUG


class TestUserRest:
    def test_get_user_self(self, httpx_mock: HTTPXMock):

        session_uuid = uuid.UUID('2d94624a-0a3d-45c6-88ee-ecbf3ff3a980')

        httpx_mock.add_response(
            method="GET",
            url='https://wappsto.com/services/2.1/user/me',
            match_headers={
                'x-session': str(session_uuid),
                "Accept": "application/json",
                "Content-type": "application/json",
            },
            json={
                'first_name': 'Tom',
                'meta': {
                    'id': str(uuid.uuid4()),
                    'type': 'user',
                }
            },  # The Reply
        )

        rest = WappstoRest(
            env=WappstoEnv.PROD,
            session=session_uuid,
        )

        reply = rest.service(WappstoService.USER).service_id('me').get()

        assert reply.first_name == 'Tom'
        # assert reply.first_name == 'Tom'

    def test_get_user_by_name(self, httpx_mock: HTTPXMock):        
        session_uuid = uuid.UUID('2d94624a-0a3d-45c6-88ee-ecbf3ff3a980')

        httpx_mock.add_response(
            method="GET",
            url='https://admin.wappsto.com/services/2.1/user?this_first_name=%3DTom',
            match_headers={
                'x-session': str(session_uuid),
                "Accept": "application/json",
                "Content-type": "application/json",
            },
            json={
                'first_name': 'Tom',
                'meta': {
                    'id': str(uuid.uuid4()),
                    'type': 'user',
                }
            },  # The Reply
        )

        rest = WappstoRest(
            env=WappstoEnv.PROD,
        )
        rest.set_session(
            session=session_uuid,
            admin=True,
        )

        reply = rest.service(
            WappstoService.USER
        ).filter(
            WappstoPath.User.first_name == 'Tom'
        ).admin().get()

        assert reply.first_name == 'Tom'
        # assert reply.first_name == 'Tom'



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:35:37+00:00

from __future__ import annotations

from enum import Enum
from typing import Any, Optional, Union

from pydantic import BaseModel


class Jsonrpc(Enum):
    field_2_1 = '2.1'


class Error(BaseModel):
    code: int
    message: str
    data: Any | None = None


class RpcError(BaseModel):
    id: str | int | None
    jsonrpc: Jsonrpc
    error: Error



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:28:46+00:00

from __future__ import annotations

import uuid
from enum import Enum

from pydantic import BaseModel, Extra, conint, constr


class NetworkItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: list[uuid.UUID] | uuid.UUID | None = None


class FactoryResetItem(BaseModel):
    class Config:
        extra = Extra.forbid

    reset_manufacturer: bool | None = None
    reset_owner: bool | None = None


class BillHistoricalEnum(Enum):
    owner = 'owner'
    manufacturer = 'manufacturer'
    creator_manufacturer = 'creator_manufacturer'


class PointEnum(Enum):
    dynamic = 'dynamic'


class DailyLimitItem(BaseModel):
    class Config:
        extra = Extra.forbid

    point: conint(ge=0) | PointEnum | None = None
    iot_traffic: conint(ge=0) | None = None
    request: conint(ge=0) | None = None
    request_time: conint(ge=0) | None = None


class MetaItem(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Creator(BaseModel):
    class Config:
        extra = Extra.allow

    network:  NetworkItem | None = None
    quantity: conint(ge=1) | None = None
    manufacturer_as_owner: bool | None = None
    factory_reset:  FactoryResetItem | None = None
    test_mode: bool | None = None
    product: str | None = None
    bill_owner: bool | None = None
    bill_user: uuid.UUID | constr(
        regex=r'(?:[a-zA-Z0-9!#$%&\'*+/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&\'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?\.)+[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?|\[(?:(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9]))\.){3}(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9])|[a-zA-Z0-9-]*[a-zA-Z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])'
    ) | None = None
    bill_historical: BillHistoricalEnum | uuid.UUID | constr(
        regex=r'(?:[a-zA-Z0-9!#$%&\'*+/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&\'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?\.)+[a-zA-Z0-9](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?|\[(?:(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9]))\.){3}(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9])|[a-zA-Z0-9-]*[a-zA-Z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])'
    ) | None = None
    daily_limit:  DailyLimitItem | None = None
    kickoff: bool | None = None
    meta:  MetaItem | None = None



# generated by datamodel-codegen:
#   filename:  <stdin>
#   timestamp: 2023-02-17T08:27:24+00:00

from __future__ import annotations

import uuid
from enum import Enum

from pydantic import BaseModel, Extra, Field, conint, constr


class Meta(BaseModel):
    class Config:
        extra = Extra.allow

    id: uuid.UUID | None = None
    trace: str | None = None
    redirect: constr(regex=r'^[0-9a-zA-Z_-]+$', min_length=1, max_length=200) | None = None
    icon: str | None = None
    tag: list[constr(min_length=2, max_length=20)] | None = None
    tag_by_user: list[constr(min_length=2, max_length=20)] | None = None
    name_by_user: constr(max_length=100) | None = None


class Type(Enum):
    one_time = 'one_time'
    subscription = 'subscription'


class Interval(Enum):
    day = 'day'
    week = 'week'
    month = 'month'
    year = 'year'


class Recurring(BaseModel):
    class Config:
        extra = Extra.forbid

    interval: Interval
    interval_count: conint(ge=1)


class Tax(BaseModel):
    id: uuid.UUID | None = None


class ApplicationProduct(BaseModel):
    class Config:
        extra = Extra.allow

    meta: Meta | None = Field(None, title='meta-2.1')
    name: str
    description: str | None = None
    active: bool | None = None
    type: Type
    currency: str
    amount: conint(ge=1)
    recurring: Recurring | None = None
    tax: Tax | None = None
    trial_period_days: conint(ge=1) | None = None



import uuid
import json
import logging

from pytest_httpx import HTTPXMock

from wappstorest import WappstoEnv
from wappstorest import WappstoPath
from wappstorest import WappstoSchema
from wappstorest import WappstoRest
from wappstorest import WappstoService


logging.root.level = logging.DEBUG


class TestSessionRest:
    def test_create(self, httpx_mock: HTTPXMock):
        session_id: str = str(uuid.uuid4())

        httpx_mock.add_response(
            method="POST",
            url='https://wappsto.com/services/2.1/session',
            match_headers={
                "Accept": "application/json",
                "Content-type": "application/json",
            },
            match_content=json.dumps({
                'username': 'Tom',
                'password': 'the_password_1234'
            }).encode(),
            json={
                'username': 'Tom',
                'meta': {
                    'id': session_id,
                    'type': 'user',
                }
            },  # The Reply
        )

        rest = WappstoRest(
            env=WappstoEnv.PROD
        )

        reply = rest.service(
            WappstoService.SESSION
        ).create(
            data=WappstoSchema.create.Session(
                username='Tom',
                password='the_password_1234'
            )
        )

        assert type(reply) == WappstoSchema.response.User
        assert reply.username == 'Tom'
        assert str(reply.meta.id) == session_id

    def test_get(self, httpx_mock: HTTPXMock):
        session_id: str = str(uuid.uuid4())
        count = 10
        httpx_mock.add_response(
            method="GET",
            url='https://wappsto.com/services/2.1/session',
            match_headers={
                "Accept": "application/json",
                "Content-type": "application/json",
                "x-session": session_id,
            },
            json={
              "count": count,
              "more": False,
              "limit": 1000,
              "meta": {
                "type": "idlist",
                "version": "2.1"
              },
              "child": [
                {
                  "type": "session",
                  "version": "2.1"
                }
              ],
              "id": [
                session_id,
              ] + [str(uuid.uuid4()) for _ in range(count)]
            },  # The Reply
        )

        rest = WappstoRest(
            env=WappstoEnv.PROD
        )
        rest.set_session(session_id)

        reply = rest.service(
            WappstoService.SESSION
        ).get()

        assert type(reply) == WappstoSchema.response.IdList
        assert session_id in reply.id
        assert reply.count == count
